//
//  AppDelegate.m
//  Channels
//

#import "AppDelegate.h"
#import "Channel.h"

@implementation AppDelegate

@synthesize window = _window;

- (void)dealloc
{
    [buttons release];
    [super dealloc];
}

- (void)windowDidResize:(NSNotification *)notification {
    //  re-arrange buttons after window resizes,
    //  not before (windowWillResize)
    [self arrangeButtons];
    //
    //  TODO:
    //    need to resize the chat windows and nick lists
    //
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // Insert code here to initialize your application
    buttons = [[NSMutableArray alloc] init ];
    
    [_window setDelegate:self];
    
    windowSize = _window.frame.size;
    contentSize = [[_window contentView] frame].size;

    barWidth = contentSize.width;
    barHeight = 20.0f;
    activeButton = -1;
    nicklist_width = 100.0f;
    [self addChannelByName:@"Console"];
}

- (void)listChannels {
    NSLog(@"%lu buttons in array", [buttons count]);
    
    for (Channel *chan in buttons) {
        NSLog(@"Button title = %@", chan.name);
    }
}

- (void)arrangeButtons {    
    NSLog(@"Arranging Buttons...");
    float bWidth;
    int numButtons;
    float xOff;
    float minButtonSize;

    // Heights are goofed up when re-sizing window
    contentSize = [[_window contentView] frame].size;
    barWidth = contentSize.width;

    // minimum width a button can be
    minButtonSize = 140;
    // how many buttons are there
    numButtons = (int)[buttons count];
    // how wide is each one going to be
    bWidth = contentSize.width / numButtons;
    
    // limit to 150 wide
    if (bWidth > minButtonSize) {
        bWidth = minButtonSize;
    }
    
    // start at 0
    xOff = 0.0;
    for (Channel *chan in buttons) {
        NSRect buttonFrame;
//        NSLog(@"%@", chan);
//        NSLog(@"bWidth = %f, xOff = %f, title = %@", bWidth, xOff, [chan buttonTitle]);
        
        // size our button
        buttonFrame.size.width = bWidth;
        buttonFrame.size.height = barHeight;
        buttonFrame.origin.x = xOff;
        buttonFrame.origin.y = contentSize.height - barHeight;
        
        [chan listNicks];
        [chan moveButton:buttonFrame];
        
        if ([buttons objectAtIndex:activeButton] == chan) {
            [chan.button setBordered:FALSE];
        }
        else {
            [chan.button setBordered:TRUE];
        }

        xOff += bWidth;
    }
    [tab selectTabViewItemAtIndex:activeButton];
    NSLog(@"activeButton = %d", activeButton);
        
    NSLog(@"Done arranging");
}

- (IBAction)addChannel:(id)sender {
    if ([[inputBox stringValue ] isEqualToString: @""])
        [errorMessage setStringValue: @"You must enter a channel name"];
    else {
        [self addChannelByName:inputBox.stringValue];
    }
}

- (IBAction)addChannelByName:(NSString *)name {
    if ([name isEqualToString:@""])
        NSLog(@"You must enter a channel name");
    else {
        NSLog(@"Adding channel %@", name);
        [errorMessage setStringValue: @""];
        
        NSView *superview = [_window contentView];
        
        // create and add the channel
        Channel *newChannel;
        newChannel = [[Channel alloc] init];
        newChannel.name = name;
        [buttons addObject:newChannel];
        [newChannel release];
        
        // add a new nickname to the list - DEBUGGING
        [newChannel addNick:newChannel.name];
        
        // setup the actual button
        NSRect buttonFrame;
        // this is just for init, as there is a method
        // that re-sizes all buttons after adding/removing one
        buttonFrame.size.width = 150;
        buttonFrame.size.height = barHeight;
        buttonFrame.origin.x = 0;
        buttonFrame.origin.y = contentSize.height - barHeight;
        
        // make sure we dealloc this when we remove it or close program
        newChannel.button = [[NSButton alloc] initWithFrame:buttonFrame];
        
        [newChannel.button setBordered:TRUE];
        newChannel.button.title = name;
        [[newChannel button] setAction:@selector(switchChannel:)];
        // button goes into the window, not the tabview like the rest
        [superview addSubview: newChannel.button];
                
        // setup the chat window
        NSRect textViewFrame;        
        textViewFrame = [tab frame];        // base dimensions on the xib tab view
        textViewFrame.size.width -= nicklist_width;        // leave room for the nicklist
        textViewFrame.origin.x = 0;
        textViewFrame.origin.y = 0;
        
        NSLog(@"textViewFrame = %f, %f, %f, %f", textViewFrame.size.width, textViewFrame.size.height,
              textViewFrame.origin.x, textViewFrame.origin.y);
        
        // make sure we dealloc this when we remove it or close program
        newChannel.chatWindow = [[NSTextView alloc] initWithFrame:textViewFrame];
        [newChannel.chatWindow setString:name];
        
        // create the tab view item to hold our UI elements
        NSTabViewItem *newTab;
        newTab = [[NSTabViewItem alloc] init];
        [newTab setLabel:name];
        // Add the tabviewitem then add chat window to it
        // 'tab' is created in IB - hard to see because it's tabless but it's there.
        [tab addTabViewItem:newTab];
        [[newTab view] addSubview:newChannel.chatWindow];
        
        // setup the nick list
        // first make a scrollview to hold the nick list

        NSRect nickScrollViewFrame;
        
        nickScrollViewFrame = [tab frame];      // start with the whole tab view
        nickScrollViewFrame.size.width = nicklist_width;    // but make it a lot less wide
        nickScrollViewFrame.origin.x = [tab frame].size.width - nicklist_width; // adjust the origin as well
        nickScrollViewFrame.origin.y = 0;
        
        newChannel.nickScrollView = [[NSScrollView alloc] initWithFrame:nickScrollViewFrame];
        [newChannel.nickScrollView setHasVerticalScroller:YES];
        [newChannel.nickScrollView setHasHorizontalScroller:YES];
        [newChannel.nickScrollView setBorderType:NSNoBorder];
        [newChannel.nickScrollView setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable];
        [newChannel.nickScrollView setBackgroundColor:[NSColor blueColor]];

        NSLog(@"nickScrollViewFrame = %f, %f, %f, %f", nickScrollViewFrame.size.width, nickScrollViewFrame.size.height,
              nickScrollViewFrame.origin.x, nickScrollViewFrame.origin.y);

        // setup the nick list

// I've commented this out because I'm just using the scrollview rect for the tableview also
        
        
//        NSRect nickListViewFrame;
//        nickListViewFrame = [tab frame];
/*        nickListViewFrame.size.height = 50;
        nickListViewFrame.size.width = 50;
        nickListViewFrame.origin.x = [tab frame].size.width - nicklist_width;
        nickListViewFrame.origin.y = 0;
        
        NSLog(@"nickListViewFrame = %f, %f, %f, %f", nickListViewFrame.size.width, nickListViewFrame.size.height,
              nickListViewFrame.origin.x, nickListViewFrame.origin.y);
*/
        // make sure we dealloc this when we remove it or close program
        newChannel.nickList = [[NSTableView alloc] initWithFrame:nickScrollViewFrame];
        [newChannel.nickList setDelegate:newChannel];
        [newChannel.nickList setDataSource:newChannel];
        [newChannel.nickList setBackgroundColor:[NSColor grayColor]];

        // you have to add a column to the tableview or it will never show up
        NSTableColumn* column = [[NSTableColumn alloc] initWithIdentifier:@"column"];
        [[column headerCell] setStringValue:@"Nicks"];
        [newChannel.nickList addTableColumn:column];
        //
        // The scrollview isn't accepting the nicklist tableview for some reason ??
        //
        // add the scroll view to the tab, then the nicklist table to the scroll view
        [newChannel.nickScrollView setDocumentView:newChannel.nickList];
        [[newTab view] addSubview:newChannel.nickScrollView];
//        [[newTab view] addSubview:newChannel.nickList];

        activeButton = (int)[buttons count]-1;
    }
    
    NSLog(@"Tabview has %lu items", [tab numberOfTabViewItems]);
    
    [self arrangeButtons];
}

- (IBAction)removeChannel:(id)sender {
    Channel *chan;
    NSTabViewItem *t;
    
    if (activeButton > 0) {
        NSLog(@"ActiveButton = %d", activeButton);
        
        chan = [buttons objectAtIndex:activeButton];
        t = [tab tabViewItemAtIndex:activeButton];

        NSLog(@"Removing channel %@", [chan.button title]);        
        [chan.button removeFromSuperview];
        [chan.chatWindow removeFromSuperview];
        [chan.nickScrollView removeFromSuperview];
        [chan.nickList removeFromSuperview];
        
        [buttons removeObjectAtIndex:activeButton];
        [tab removeTabViewItem:t];

        if ([buttons count] == 0) {
            activeButton = -1;
        }
        else {
            activeButton = (int)[buttons count]-1;
        }
        NSLog(@"Active channel is now %d", activeButton);
        [self arrangeButtons];
    }
    else {
        NSLog(@"Sorry, can't delete the last channel");
    }
}

- (IBAction)switchChannel:(id)sender {
    int whichButton = [self findButtonWithTitle:[sender title]];
    NSLog(@"You clicked button %d titled %@",
          whichButton,
          [sender title]);
    
    // change state of button that was clicked, and reset state of all others
    for (Channel *chan in buttons) {
        if (chan == [buttons objectAtIndex:whichButton]) {
            [chan.button setBordered:FALSE];
            // show the tab associated with that button
            [tab selectTabViewItemAtIndex:whichButton];
            NSLog(@"switchChannel: Showing tab %d", whichButton);
            activeButton = whichButton;
        }
        else {
            [chan.button setBordered:TRUE];
        }
    }
}

- (int)findButtonWithTitle:(NSString *)title {
    int index = 0;
    
    for (Channel *c in buttons) {
        if ([c.button.title isEqualToString:title]) {
            return index;
        }
        index++;
    }
    
    return -1;
}

@end
